#include "StdAfx.h"
#include "PartitionImageShape.h"
#include "NControl.h"

#include "ScriptletConstant.h"

REGISTER_COMPONENT_CREATOR(SCRIPTLET_PARTITIONIMAGE_NODE, CPartitionImageShape);
CPartitionImageShape::CPartitionImageShape(void)
{
	m_nPartiton = 1;
	m_pParentControl = NULL;
	m_bSelectVisible = FALSE;
}

CPartitionImageShape::~CPartitionImageShape(void)
{
}
void CPartitionImageShape::SetPartition(int part)
{
	m_nPartiton = max(1, part);
}
void CPartitionImageShape::SetSelectVisible(BOOL bVisible)
{
	m_bSelectVisible = bVisible;
}
void CPartitionImageShape::OnAddNotify()
{
	__super::OnAddNotify();

	CComponent* statableCompoent = CStateControl::FindStatable(this);
	if (statableCompoent != NULL)
	{
		CNControl* pControl = dynamic_cast<CNControl*>(statableCompoent);
		m_pParentControl = pControl;
		m_stateControl.AttachControl(pControl);		
	}
}
void CPartitionImageShape::OnRemoveNotify()
{
	__super::OnRemoveNotify();
	m_stateControl.DetachControl();
	m_pParentControl = NULL;
}
void CPartitionImageShape::DrawComponent(HDC hDC)
{
	if (m_bSelectVisible && m_stateControl.IsPushed() == FALSE)
	{
		return;
	}

	CRect rcClient;
	GetDrawingBounds(&rcClient);

	DrawPartImage(hDC, rcClient);
}
int CPartitionImageShape::GetPartIndex()
{
	if (m_pParentControl != NULL)
	{
		if (m_pParentControl->IsWindowEnabled() == FALSE)
		{
			return E_PART_DISABLE;
		}
	}	
	if (m_stateControl.IsPushed())
	{
		return GetPushedPartIndex();
	}
	return GetUnPushedPartIndex();
}
int CPartitionImageShape::GetPushedPartIndex()
{
	switch (m_stateControl.GetState())
	{
	case CStateControl::E_CONTROLSTATE_NORMAL:
		return E_PART_PRESS;
	case CStateControl::E_CONTROLSTATE_DOWN:
		return E_PART_PRESS;
	case CStateControl::E_CONTROLSTATE_OVER:
		return E_PART_PRESS;
	default:
		return E_PART_PRESS;
	}
}
int CPartitionImageShape::GetUnPushedPartIndex()
{
	switch (m_stateControl.GetState())
	{
	case CStateControl::E_CONTROLSTATE_NORMAL:
		return E_PART_NORMAL;
	case CStateControl::E_CONTROLSTATE_DOWN:
		return E_PART_PRESS;
	case CStateControl::E_CONTROLSTATE_OVER:
		return E_PART_OVER;
	default:
		return E_PART_NORMAL;
	}
}
BOOL CPartitionImageShape::GetOrginPreferredSize(LPSIZE lpSize)
{
	BOOL bSuccess = __super::GetOrginPreferredSize(lpSize);
	if (bSuccess && lpSize->cx > 0 && m_nPartiton > 1)
	{
		lpSize->cx = lpSize->cx / m_nPartiton;
	}
	return bSuccess;
}

void CPartitionImageShape::DrawPartImage( HDC hDC, CRect rcClient )
{
	int partIndex = min(m_nPartiton-1, GetPartIndex());
	if (m_pImage)
	{
		int partImageWidth = m_pImage->GetWidth() / m_nPartiton;
		CRect rcBound(partIndex * partImageWidth, 0, (partIndex + 1) * partImageWidth, m_pImage->GetHeight());
		
		m_pImage->DrawImage(hDC, &rcBound, &rcClient);
	}
}

REGISTER_COMPONENT_CREATOR(SCRIPTLET_PARTITIONTILEIMAGE_NODE, CPartitionTileImageShape);
void CPartitionTileImageShape::DrawComponent(HDC hDC)
{
	if (m_pImage)
	{
		int nWidthOfImage = m_pImage->GetWidth();
		int nHeightOfImage = m_pImage->GetHeight();
		if (nHeightOfImage > 0 && nWidthOfImage > 0)
		{
			CRect rcBound(0, 0, nWidthOfImage, nHeightOfImage);
			CRect rcClient;
			GetDrawingBounds(&rcClient);

			int nCountOfHorizontal = rcClient.Width() / nWidthOfImage + 0;
			int nCountOfVertical = rcClient.Height() / nHeightOfImage + 0;

			CRect rcImage;
			for (int i=0; i<nCountOfHorizontal; i++)
			{
				for (int j=0; j<nCountOfVertical; j++)
				{
					rcImage.left = i * nWidthOfImage;
					rcImage.top = j * nHeightOfImage;
					rcImage.right = rcImage.left + nWidthOfImage;
					rcImage.bottom = rcImage.top + nHeightOfImage;

					DrawPartImage(hDC, &rcImage);
				}
			}			
		}		
	}
}